import os
from flask import current_app, url_for
from itsdangerous import (
    BadTimeSignature,
    SignatureExpired,
    URLSafeTimedSerializer,
)
from dotenv import load_dotenv

from splent_io.splent_feature_confirmemail.repositories import ConfirmemailRepository
from splent_framework.services.BaseService import BaseService

# Load environment variables
load_dotenv()


class ConfirmemailService(BaseService):
    def __init__(self):
        super().__init__(ConfirmemailRepository())
        self.repository = ConfirmemailRepository()
        self.CONFIRM_EMAIL_SALT = os.getenv("CONFIRM_EMAIL_SALT", "sample_salt")
        self.CONFIRM_EMAIL_TOKEN_MAX_AGE = os.getenv(
            "CONFIRM_EMAIL_TOKEN_MAX_AGE", 3600
        )

    def get_serializer(self):
        return URLSafeTimedSerializer(current_app.config["SECRET_KEY"])

    def get_token_from_email(self, email):
        s = self.get_serializer()
        return s.dumps(email, salt=self.CONFIRM_EMAIL_SALT)

    def send_confirmation_email(self, user_email):
        token = self.get_token_from_email(user_email)
        url = url_for("confirmemail.confirm_user", token=token, _external=True)

        html_body = f"<a href='{url}'>Please confirm your email</a>"

        mail_service = current_app.mail_service
        mail_service.send_email(
            "Please confirm your email",
            recipients=[user_email],
            body="Please confirm your email by clicking the link below.",
            html_body=html_body,
        )

    def confirm_user_with_token(self, token):
        s = self.get_serializer()
        try:
            email = s.loads(
                token,
                salt=self.CONFIRM_EMAIL_SALT,
                max_age=self.CONFIRM_EMAIL_TOKEN_MAX_AGE,
            )
        except SignatureExpired:
            raise Exception("The confirmation link has expired.")
        except BadTimeSignature:
            raise Exception("The confirmation link has been tampered with.")

        # Import AuthenticationService inside the method to avoid circular imports
        from splent_io.splent_feature_auth.services import AuthenticationService

        user = AuthenticationService().get_by_email(email, active=False)
        user.active = True
        self.repository.session.commit()
        return user



"""
Signal handlers for the confirmemail feature.

Connects to auth's user_registered signal to send a confirmation email
when a new user signs up.
"""

from splent_io.splent_feature_auth.services import user_registered
from splent_io.splent_feature_confirmemail.services import ConfirmemailService


@user_registered.connect
def on_user_registered(sender, user, **kwargs):
    """Send confirmation email when a new user registers."""
    try:
        service = ConfirmemailService()
        service.send_confirmation_email(user.email)
    except Exception:
        pass  # Don't block registration if email fails



from splent_io.splent_feature_confirmemail.models import Confirmemail
from splent_framework.repositories.BaseRepository import BaseRepository


class ConfirmemailRepository(BaseRepository):
    def __init__(self):
        super().__init__(Confirmemail)



from flask import flash, redirect, render_template, url_for
from flask_login import login_user
from splent_io.splent_feature_confirmemail import confirmemail_bp


@confirmemail_bp.route("/confirmemail", methods=["GET"])
def index():
    return render_template("confirmemail/index.html")


@confirmemail_bp.route("/confirm_user/<token>", methods=["GET"])
def confirm_user(token):
    from splent_io.splent_feature_confirmemail.services import (
        ConfirmemailService,
    )  # ⬅️ IMPORT AQUÍ

    confirmemail_service = ConfirmemailService()

    try:
        user = confirmemail_service.confirm_user_with_token(token)
    except Exception as exc:
        flash(exc.args[0], "danger")
        return redirect(url_for("auth.show_signup_form"))

    login_user(user, remember=True)
    return redirect(url_for("public.index"))



from splent_framework.blueprints.base_blueprint import BaseBlueprint
from .services import ConfirmemailService

confirmemail_bp = BaseBlueprint("confirmemail", __name__, template_folder="templates")
confirmemail_service = ConfirmemailService()


def init_feature(app):
    app.confirmemail_service = confirmemail_service


def inject_context_vars(app):
    return {}


# Register signal handlers (send confirmation on user registration)
from splent_io.splent_feature_confirmemail import signals  # noqa: F401,E402
